module decode_opcode(opcode,
	R_format,Lw, Sw, ALUOp,PCOp);
input [5:0] opcode;
output R_format, Lw, Sw;
output [7:0] ALUOp;
output [3:0] PCOp;

assign R_format = (opcode == 6'b000000);

/*
*   100011  load word
*   100000  load byte, sign extend
*   100100  load byte, zero extend
*   101000  store byte
*   101011  store word
*/

assign Lw = (opcode[5]? ~opcode[3] : 1'b0);
assign Sw = (opcode[5]?  opcode[3] : 1'b0);

// immediate instructions
// ALUOp[7] flags an immediate instruction
// ALUOp[6] flags an arithmetic instruction (sign extension required)
reg [7:0] ALUOp;
always 
case (opcode)
6'b001000: ALUOp <= 8'b11100000; // add signed
6'b001001: ALUOp <= 8'b11100001; // add unsigned
6'b001010: ALUOp <= 8'b11101010; // slt
6'b001100: ALUOp <= 8'b10100100; // AND
6'b001101: ALUOp <= 8'b10100101; // OR
6'b001110: ALUOp <= 8'b10100110; // XOR
default:   ALUOp <= 8'b00xxxxxx;
endcase

reg [3:0] PCOp;
always
case (opcode)
// branch  10xx
6'b000100: PCOp <= 4'b1000; // beq
6'b000101: PCOp <= 4'b1001; // bne
6'b000001: PCOp <= 4'b1010; // bltz and bgez
// jump 11xx		
6'b000010: PCOp <= 4'b1101; // j  (jump)
6'b000011: PCOp <= 4'b1111; // jal
default:   PCOp <= 4'b0xxx;
endcase

endmodule