module srisc(clock,reset,pc,npc,instruction,ds,dt,dd,out,wpc,mem_op,RegWrite);
parameter 
	L = 24,
	LC = 6;

`define immed_op ALUOp[7]
input clock, reset;
output [31:0] instruction;
output [LC-1:0] npc, pc, wpc;

output [L-1:0] ds, dt, dd;
output [L-1:0] out;
output RegWrite;
output mem_op;
wire zero, ovfl;

wire [4:0] shift;


wire R_format, Lw, Sw;
wire [7:0] ALUOp;
wire [3:0] PCOp;
wire [5:0] func;

wire [5:0] opcode,fn;
wire [4:0] rs,rt,rd;
wire [31:0] immed;
wire [15:0] se;
wire [25:0] jta;

reg [LC-1:0] pc;

parameter is_reset = 2'b00, 
		decode = 2'b01,
		   exe = 2'b10,
		 store = 2'b11;
		
reg [1:0] state;

wire is_decode, is_exe;

assign is_decode = (state==decode);
assign is_exe = (state==exe);
assign wpc = (mem_op &&  !is_exe? pc: npc);

instr instr_cache(.address(wpc),.inclock(clock),.q(instruction));

wire [31:0] dataout;
wire write_enable;

data_cache mem2(.address(out[LC+1:2]),.inclock(clock),.we(write_enable),
	.data(dt), .q(dataout));
assign write_enable = (state==store);

assign opcode = instruction[31:26];
assign rs = instruction[25:21];
assign rt = instruction[20:16];
assign rd = instruction[15:11];
assign shift = instruction[10:6];
assign fn = instruction[5:0];
assign jta = instruction[25:0];

wire lui;
wire [31:0] imm, upper;
assign immed[15:0] = instruction[15:0];
assign se = (instruction[15]? 16'hFFFF: 16'h0000);
assign immed[31:16] = (ALUOp[6] || mem_op? se: 16'h0000);
assign imm = {se,immed[15:0]};
assign upper = {immed[15:0],16'h0000};
assign lui = (opcode==6'h0f); 

decode_opcode dop(opcode, R_format, Lw, Sw, ALUOp, PCOp);
	
wire branch_active;	
branch_control branch_control_unit(
	.control(PCOp[1:0]),
	.branch(branch_active),
	.zero(zero),
	.rt(instruction[16]),
	.ds(ds[L-1])
);
	
wire [1:0] bcontrol;
assign bcontrol = PCOp[3:2];

wire is_branch, branch, jump, jr, jal;
assign jal = (PCOp==4'hf);
assign jr = R_format && (fn==6'h08);
assign jump = (bcontrol[1]? bcontrol[0]: 1'b0) || jr;
assign is_branch = (bcontrol[1]? !bcontrol[0]: 1'b0);
assign branch = is_branch && branch_active;

assign RegWrite = ((R_format | jal  | `immed_op | lui) & is_decode) |
				( Lw & !is_decode);


wire [L-1:0] bt;
wire [4:0] rdm;
wire Acontrol;
//wire Acontrol, mem_op;
assign mem_op = Lw || Sw;
assign Acontrol = `immed_op || mem_op;
wire [5:0] fna, fnb;
assign fna = (mem_op? 6'h20: ALUOp[5:0]);
assign fnb = (is_branch? 6'h22: fn);

assign func = (Acontrol: fna: fnb);
assign bt = (Acontrol? immed[L-1:0] : dt);
assign rdm = (Acontrol||lui? rt : (jal? 5'h1f: rd));
alu #(.L(L)) alu_unit(ds,bt,shift,func,out,zero,ovfl);

wire [31:0] pcr;
assign pcr = {26'h000000, pc+1'b1};
assign dd = (jal? pcr[L-1:0]: Lw? dataout[L-1:0]: lui? upper[L-1:0]: out);

reg_file #(.L(L)) reg_unit(clock,reset,ds,dt,dd,rs,rt,rdm,RegWrite);


/*
*   if (jump) npc = (jr? ds : jta);
*   else if (branch) npc = pc + imm;
*	else npc  = pc + 1
*/
assign npc = (jump?  (jr? ds[LC-1:0]: jta[LC-1:0]): pc + (branch? imm[LC-1:0]: 1'b1));

always @(posedge clock or negedge reset)
if (!reset) state <= is_reset;
else case(state)
is_reset:
	begin
		pc <= 6'b111111;		
		state <= decode;
	end
decode:
	begin
		if (Lw) state <= exe;
		else if (Sw) state <= store;
		else 
		begin
			pc <= npc;
			state <= decode;
		end
	end
store: 
	state <= exe;
exe:
	begin
		pc <= npc;
		state <= decode;
	end
default:
	state <= decode;
endcase

endmodule